import { ActionError, defineAction } from 'astro:actions';
import { z } from 'astro:schema';
import { db } from '@db/database.ts';
import { getJavaUuid } from '@util/minecraft.ts';
import { getSetting, SettingKey } from '@util/settings.ts';

export const signup = {
  signup: defineAction({
    input: z.object({
      firstname: z.string().min(2),
      lastname: z.string().min(2),
      // this will be inaccurate as it is evaluated only once
      birthday: z
        .string()
        .date()
        .max(Date.now() - 1000 * 60 * 60 * 24 * 365 * 6),
      phone: z.string().nullable(),
      username: z.string(),

      teamMember: z.string(),
      teamName: z.string().nullable()
    }),
    handler: async (input) => {
      // check if signup is allowed
      if (!(await getSetting(db, SettingKey.SignupEnabled))) {
        throw new ActionError({
          code: 'FORBIDDEN',
          message: 'Die Anmeldung ist derzeit deaktiviert'
        });
      }

      // check if username and team member is equal
      if (input.username.toLowerCase() === input.teamMember.toLowerCase()) {
        throw new ActionError({
          code: 'CONFLICT',
          message: 'Du kannst nicht mit dir selber in einem Team sein'
        });
      }

      // check if the user were already signed up
      if (await db.getUserByUsername({ username: input.username })) {
        throw new ActionError({
          code: 'CONFLICT',
          message: 'Du hast dich bereits registriert'
        });
      }

      const teamMember = await db.getUserByUsername({ username: input.teamMember });
      const teamDraft = await db.getTeamDraftByMemberOne({ memberOneName: input.teamMember });

      // check if the team member already signed up but is in another team already
      if (teamMember && (!teamDraft || teamDraft.memberTwoName.toLowerCase() != input.username.toLowerCase())) {
        throw new ActionError({
          code: 'FORBIDDEN',
          message: 'Dein Teampartner ist bereits in einem anderen Team'
        });
      }

      let uuid;
      try {
        uuid = await getJavaUuid(input.username);
      } catch (_) {
        throw new ActionError({
          code: 'NOT_FOUND',
          message: `Es wurde kein Minecraft Java Account mit dem Username ${input.username} gefunden`
        });
      }

      // check if user is blocked
      if (uuid) {
        const blockedUser = await db.getBlockedUserByUuid({ uuid: uuid });
        if (blockedUser) {
          throw new ActionError({
            code: 'FORBIDDEN',
            message: 'Du bist für die Registrierung gesperrt'
          });
        }
      }

      if (!teamDraft) {
        // check if a team with the same name already exists
        if (input.teamName) {
          if (await db.getTeamByName({ name: input.teamName })) {
            throw new ActionError({
              code: 'CONFLICT',
              message: 'Es gibt bereits ein Team mit diesem Namen'
            });
          }
          // no team draft is present and `input.teamName` is not present
        } else {
          throw new ActionError({
            code: 'BAD_REQUEST',
            message: 'Es ist noch kein Team auf dich und deinen Teampartner registriert'
          });
        }
      }

      const team = await db.transaction(async (tx) => {
        const user = await tx.addUser({
          firstname: input.firstname,
          lastname: input.lastname,
          birthday: input.birthday,
          telephone: input.phone,
          username: input.username,
          uuid: uuid
        });

        let team;
        if (teamDraft) {
          team = await tx.getTeamById({ id: teamDraft.teamId });
        } else {
          team = await tx.addTeam({ name: input.teamName! });

          await tx.addTeamDraft({ memberOneName: input.username, memberTwoName: input.teamMember, teamId: team.id });
        }

        await tx.addTeamMember({ teamId: team.id, userId: user.id });

        return team;
      });

      return {
        team: {
          name: team.name,
          color: team.color
        }
      };
    }
  })
};
